/*
 * MP3/MPlayer plugin to VDR (C++)
 *
 * (C) 2001-2003 Stefan Huelswitt <huels@iname.com>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>

#include "common.h"
#include "decoder-mp3-stream.h"
#include "stream.h"

//#define DUMP_HEAD "/var/tmp/headers"

// -----------------------------------------------------------------------------

#ifdef DUMP_HEAD
void Dump(const char *name, char *buffer)
{
  FILE *f=fopen(name,"a");
  if(f) {
    fprintf(f,"<<<< %s\n",buffer);
/*
    int n=strlen(buffer);
    for(int i=0 ; i<n ; i+=8) {
      fprintf(f,"%04x: ",i);
      for(int l=0 ; l<8 && i+l<n ; l++) fprintf(f,"%02x ",buffer[i+l]);
      fprintf(f,"\n");
      }
*/
    fclose(f);
    }
}
#endif

// --- cNetScanID3 -------------------------------------------------------------

class cNetScanID3 : public cScanID3 {
private:
  cNetStream *nstr;
public:
  cNetScanID3(cNetStream *Str, bool *Urgent);
  virtual bool DoScan(bool KeepOpen=false);
  virtual void InfoHook(struct mad_header *header);
  };

cNetScanID3::cNetScanID3(cNetStream *Str, bool *Urgent)
:cScanID3(Str,Urgent)
{
  nstr=Str;
}

bool cNetScanID3::DoScan(bool KeepOpen)
{
  Clear();
  const char *s=nstr->IcyName();
  if(s) Title=strdup(s); else FakeTitle(nstr->Filename);
  s=nstr->IcyUrl();
  if(s) Album=strdup(s);
  Total=0;
  ChMode=3;
  DecoderID=DEC_MP3S;
  InfoDone();
  return true;
}

void cNetScanID3::InfoHook(struct mad_header *header)
{
  SampleFreq=header->samplerate;
  Channels=MAD_NCHANNELS(header);
  ChMode=header->mode;
 
  int br=header->bitrate;
  if(Bitrate<0) Bitrate=br;
  else if(Bitrate!=br) {
    if(MaxBitrate<0) {
      if(Bitrate<br) MaxBitrate=br;
      else { MaxBitrate=Bitrate; Bitrate=br; }
      }
    else {
      if(br>MaxBitrate) MaxBitrate=br;
      if(br<Bitrate)    Bitrate=br;
      }
    }
}

// --- cMP3StreamDecoder -------------------------------------------------------

cMP3StreamDecoder::cMP3StreamDecoder(const char *Filename)
:cMP3Decoder(Filename,false)
{
  nstr=new cNetStream(filename);
  str=nstr;
  nscan=new cNetScanID3(nstr,&urgentLock);
  scan=nscan;
  isStream=true;
}

bool cMP3StreamDecoder::Valid(void)
{
  bool res=false;
  if(TryLock()) {
    if(nstr->Valid()) res=true;
    Unlock();
    }
  return res;
}
